# Generated by Django 1.9.7 on 2016-06-07 11:22
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("modeladmintest", "0003_publisher"),
    ]

    operations = [
        migrations.CreateModel(
            name="VenuePage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.Page",
                    ),
                ),
                ("address", models.CharField(max_length=300)),
                ("capacity", models.IntegerField()),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]
